﻿namespace Tracer

module Camera =
    type Point = Point.Point
    type Vector = Vector.Vector
    type Ray = Ray.Ray

    type Camera = | C of Point * Point * Vector * float * float * float * int * int

    let mkCamera position lookat up distance unitWidth unitHeight pixelWidth pixelHeight=
        //let x = if Vector.getX up < 0.0 then 0.0 else Vector.getX up
        //let y = if Vector.getY up < 0.0 then 0.0 else Vector.getY up
        //let z = if Vector.getZ up < 0.0 then 0.0 else Vector.getZ up
        //let upV = Vector.mkVector x y z 
        //C(position, lookat, upV, distance, unitWidth, unitHeight, pixelWidth, pixelHeight)
        C(position, lookat, up, distance, unitWidth, unitHeight, pixelWidth, pixelHeight)

    let getPosition (C(p,_,_,_,_,_,_,_)) = p

    let getDistance (C(_,_,_,d,_,_,_,_)) = d

    let getPixelHeight (C(_,_,_,_,_,_,_,ph)) = ph
    
    let getPixelWidth (C(_,_,_,_,_,_,pw,_)) = pw 

    let getLookAt (C(_,la,_,_,_,_,_,_)) = la


    let getRay (C(p,q,u,z,w,h,x,y)) a b =
        let i = Vector.normalise (Point.distance p q)
        let r = Vector.normalise (Vector.crossProduct u i)
        let d = Vector.normalise (Vector.crossProduct r i)
        let p1 = Point.move p (z*i)
        let p2 = Point.move p1 ((h/2.0)*u)
        let p3 = Point.move p2 ((-w/2.0)*r)
        let W = w/(float)x
        let H = h/(float)y
        let pr = Point.move p3 ((((float)a+0.5)*W)*r)
        let pd = Point.move pr ((((float)b+0.5)*H)*d)
        let dv = (Vector.normalise(Point.distance p pd))
        Ray.mkRay p dv

    let getRays(C(p,q,u,z,w,h,px,py)) = 
        let i = Vector.normalise (Point.distance p q)
        let r = Vector.normalise (Vector.crossProduct u i)
        let d = Vector.normalise (Vector.crossProduct r i)
        let p1 = Point.move p (z*i)
        let p2 = Point.move p1 ((h/2.0)*u)
        let p3 = Point.move p2 ((-w/2.0)*r)
        let W = w/(float)px
        let H = h/(float)py

        [| for x in 0..(px - 1) -> 
            [| for y in 0..(py - 1) ->
                let pr = Point.move p3 ((((float)x + 0.5) * W) * r)
                let pd = Point.move pr ((((float)y + 0.5) * H) * d)
                let dv = Vector.normalise(Point.distance p pd)
                Ray.mkRay p dv
            |]
        |]

