﻿namespace Tracer

module Color =
    type Color = | Clr of float * float * float    
    let gamma = 2.0
    let mkColor r g b = Clr(r,g,b)

    let getR (Clr(r,_,_)) = r
    let getG (Clr(_,g,_)) = g
    let getB (Clr(_,_,b)) = b
    let getRGB (Clr(r,g,b)) = (r, g, b)

    let addColor c1 c2 =
        let r1, g1, b1 = getRGB c1
        let r2, g2, b2 = getRGB c2
        Color.Clr(r1+r2, g1 + g2, b1 + b2)

    let scaleColor (Clr(r,g,b)) s =
        Clr((float)(r*s), (float)(g*s), (float)(b*s))

    let cap x y = 
        if x > y then y 
        elif x < 0.0 then 0.0
        else x
    let capColor c = 
        let r, g, b = cap (getR c) 255.0, cap (getG c) 255.0, cap (getB c) 255.0
        Clr(r, g, b)

    let scaleTo1 (Clr(r,g,b)) = 
        let r1 = cap (r/255.0) 1.0
        let g1 = cap (g/255.0) 1.0
        let b1 = cap (b/255.0) 1.0
        Clr(r1, g1, b1)

    let addColors = List.fold (fun acc element -> addColor element acc) (mkColor 0.0 0.0 0.0)
    let capAddColors l = 
        let c = addColors l
        capColor c

    let avgFromList l = scaleColor (addColors l) ((float)(1 / l.Length)) //add all the colors together to one color, and scale that by doing the equivalent of dividing by the number of colors we'd added

    let toSysColorGammaCorrect c = 
        let gammaCorrect255Color c =
            let r, g, b = getRGB c
            //System.Drawing.Color.FromArgb (255, int (255.0*(System.Math.Pow(r, 1.0/gamma))), int (255.0*(System.Math.Pow(g, 1.0/gamma))), int (255.0*(System.Math.Pow(b, 1.0/gamma))))
            let gc old = int (255.0 * ((old/255.0)**gamma)) //gc stands for gammacorrect
            System.Drawing.Color.FromArgb(gc r, gc g, gc b)
        gammaCorrect255Color c
    let toSysColor c = 
        let r, g, b = getRGB c
        System.Drawing.Color.FromArgb(int r, int g, int b)
    let fromSysColor (c:System.Drawing.Color) = Clr((float)c.R, (float)c.G, (float)c.B)